class SpellCheckerParameter:

    __threshold: float
    __deMiCheck: bool
    __rootNGram: bool
    __minWordLength: int
    __domain: str

    def __init__(self):
        """
        Constructs a SpellCheckerParameter object with default values.
        The default threshold is 0.0, the De-Mi check is enabled, the root ngram is enabled and
        the minimum word length is 4.
        """
        self.__threshold = 0.0
        self.__deMiCheck = True
        self.__rootNGram = True
        self.__minWordLength = 4
        self.__domain = ""

    def setThreshold(self, threshold: float):
        """
        Sets the threshold value used in calculating the n-gram probabilities.
        :param threshold: the threshold for the spell checker
        """
        self.__threshold = threshold

    def setDeMiCheck(self, deMiCheck: bool):
        """
        Enables or disables De-Mi check for the spell checker.
        :param deMiCheck: a boolean indicating whether the De-Mi check should be enabled (true) or disabled (false)
        """
        self.__deMiCheck = deMiCheck

    def setRootNGram(self, rootNGram: bool):
        """
        Enables or disables the root n-gram for the spell checker.
        :param rootNGram: a boolean indicating whether the root n-gram should be enabled (true) or disabled (false)
        """
        self.__rootNGram = rootNGram

    def setMinWordLength(self, minWordLength: int):
        """
        Sets the minimum length of words viable for spell checking.
        :param minWordLength: the minimum word length for the spell checker
        """
        self.__minWordLength = minWordLength

    def setDomain(self, domain: str):
        """
        Sets the _domain name to the specified value.
        :param domain: the new domain name to set for this object
        """
        self.__domain = domain

    def getThreshold(self) -> float:
        """
        Returns the threshold value used in calculating the n-gram probabilities.
        :return: the threshold for the spell checker
        """
        return self.__threshold

    def isDeMiCheck(self) -> bool:
        """
        Returns whether De-Mi check is enabled for the spell checker.
        :return: a boolean indicating whether De-Mi check is enabled for the spell checker
        """
        return self.__deMiCheck

    def isRootNGram(self) -> bool:
        """
        Returns whether the root n-gram is enabled for the spell checker.
        :return: a boolean indicating whether the root n-gram is enabled for the spell checker
        """
        return self.__rootNGram

    def getMinWordLength(self) -> int:
        """
        Returns the minimum length of words viable for spell checking.
        :return: the minimum word length for the spell checker
        """
        return self.__minWordLength

    def getDomain(self) -> str:
        """
        Returns the domain name
        :return: the domain name
        """
        return self.__domain



from SpellChecker.TrieNode import TrieNode


class Trie:

    __root_node: TrieNode

    def __init__(self):
        """
        A constructor of {@link Trie} class which constructs a new Trie with an empty root node
        """
        self.__root_node = TrieNode()

    def insert(self, word: str):
        """
        Inserts a new word into the Trie
        :param word: The word to be inserted
        """
        current_node = self.__root_node
        for i in range(len(word)):
            ch = word[i]
            if current_node.getChild(ch) is None:
                current_node.addChild(ch, TrieNode())
            current_node = current_node.getChild(ch)
        current_node.setIsWord(True)

    def search(self, word: str) -> bool:
        """
        Checks if a word is in the Trie
        :param word: The word to be searched for
        :return: true if the word is in the Trie, false otherwise
        """
        node = self.getTrieNode(word.lower())
        if node is None:
            return False
        else:
            return node.isWord()

    def startsWith(self, prefix: str) -> bool:
        """
        Checks if a given prefix exists in the Trie
        :param prefix: The prefix to be searched for
        :return: true if the prefix exists, false otherwise
        """
        if self.getTrieNode(prefix.lower()) is None:
            return False
        else:
            return True

    def getTrieNode(self, word: str) -> TrieNode:
        """
        Returns the TrieNode corresponding to the last character of a given word
        :param word: The word to be searched for
        :return: The TrieNode corresponding to the last character of the word
        """
        current_node = self.__root_node
        for i in range(len(word)):
            ch = word[i]
            if current_node.getChild(ch) is None:
                return None
            current_node = current_node.getChild(ch)
        return current_node



from Dictionary.Word import Word
from SpellChecker.Operator import Operator


class Candidate(Word):

    __operator: Operator

    def __init__(self,
                 candidate: str,
                 operator: Operator):
        """
        Constructs a new Candidate object with the specified candidate and operator.
        :param candidate: The word candidate to be checked for spelling.
        :param operator: The operator to be applied to the candidate in the spell checking process.
        """
        super().__init__(candidate)
        self.__operator = operator

    def getOperator(self) -> Operator:
        """
        Returns the operator associated with this candidate.
        :return: The operator associated with this candidate.
        """
        return self.__operator
